function state = cart_pole_sba( state, action )
% state = cart_pole_sba( state, action )
% 
%     Takes as input a state and action and returns the next state
%   TAU seconds later. The state is defined to be a column vector
%   according to:
%
%   state = [ x           <- the cart position
%             x_dot       <- the cart velocity
%             theta       <- the angle of the pole
%             theta_dot ] <- the angular velocity of the pole.
%
%   The action is force applied to the cart and can be positive or
%   negative. All units are standard SI: meters, meter/second,
%   radians, radians/second, and Newtons. Environment parameters,
%   such as gravity and friction, can be changed within the script
%   defining this function (cart_pole.m).
% 
%   Notes:
%    - A minor design consideration is whether to update velocity
%      changes or position changes first. In this implementation,
%      position (including angle) is updated before velocity.
%
%
% Originally written by Zhenzhen Liu
% Modified by Scott Livingston, October 2007
%
% MODIFIED by Scott Livingston, December 2008
%   to use dynamics presented in [1].
%
% Machine Intelligence Lab, EECS - Univ. of Tennessee
%
% Released under the GNU General Public License, version 3, see
% GPLv3.txt or README for details.


% state components (continuous-valued)
x         = state(1); % current position (units in m)
x_dot     = state(2); % current velocity (in m/s)
theta     = state(3); % current angle of the pole (in radians)
theta_dot = state(4); % current rate of change in angle of the pole (in radians/s)

% parameters for simulation

g          =  -9.8; % gravity (units are m/s^2)
mass_cart  =  1.0; % mass of the cart (in kg)
mass_pole  =  0.1; % mass of the pole (in kg)
total_mass = mass_cart + mass_pole;
plength     =  0.5; % half of the length (in m) of the pole;
                   % (location of pole's center of mass)
tau        = 0.02; % time interval (in seconds) for updating the values
mu_c       = 0.0005;   % coefficient of friction between cart and track
mu_p       = 0.000002; % coefficient of friction at pole-cart joint.

% calculate the acceleration of theta, given the current state and action
theta_acc = (g*sin(theta) + cos(theta)*( (-action - mass_pole*plength*theta_dot*theta_dot*sin(theta) + mu_c*sign(x_dot))/(mass_cart+mass_pole) ) - mu_p*theta_dot/(mass_pole*plength)) / (plength*(4/3 - mass_pole*cos(theta)*cos(theta)/(mass_cart + mass_pole)));

% now, calculate the acceleration of x
x_acc = (action + mass_pole*plength*( theta_dot*theta_dot*sin(theta) - theta_acc*cos(theta) ) - mu_c*sign(x_dot)) / (mass_cart + mass_pole);

% update the four state variables, using Euler method
x         = x + tau*x_dot;
x_dot     = x_dot + tau*x_acc;
theta     = theta + tau*theta_dot;
theta_dot = theta_dot + tau*theta_acc;

% handle special case: pole is parallel with the horizontal axis
if theta > pi/2, theta = pi/2; theta_dot = 0; end;
if theta < -pi/2, theta = -pi/2; theta_dot = 0; end;

% return the new state
state = [ x; x_dot; theta; theta_dot ];
