function [s_ind] = state_dec( X )
% [s_ind] = state_dec( X )
%
% Decode the state vector X (assumed to be a column vector)
% into one of the intervals defined in [1], repeated here for
% convenience (all values are +/-; "infty" means infinity):
% x:         0.8, 2.4 m
% x_dot:     0.5, infty m/s
% theta:     0, 1, 6, 12 deg
% theta_dot: 50, infty deg/s
%
% NOTES: - States are ONLY defined within these intervals;
%          e.g., x = 3.0 m doesn't map to a state index.
%        - s_ind is one-indexed (i.e., numbering begins at 1).
%        - Returns -1 on error.
%        - Angles are assumed to be in radians (not degrees).
%
% Scott Livingston <slivingston@caltech.edu>
% 30 Dec 2008
% Machine Intelligence Lab, EECS Dept., Univ. of Tennessee, Knoxville
%
% Released under the GNU General Public License, version 3, see
% GPLv3.txt or README for details.

x_ints = [-2.4 -0.8 0.8 2.4];
x_dot_ints = [-Inf -0.5 0.5 Inf];
theta_ints = [-12 -6 -1 0 1 6 12]*pi/180;
theta_dot_ints = [-Inf -50 50 Inf]*pi/180;

%preload interval lengths
len_x_ints = length(x_ints);
len_x_dot_ints = length(x_dot_ints);
len_theta_ints = length(theta_ints);
len_theta_dot_ints = length(theta_dot_ints);

%check bounds
if X(1) < x_ints(1) || X(1) > x_ints(end) || X(2) < x_dot_ints(1) || X(2) > x_dot_ints(end) || X(3) < theta_ints(1) || X(3) > theta_ints(end) || X(4) < theta_dot_ints(1) || X(4) > theta_dot_ints(end)
    s_ind = -1;
    return;
end

%find appropriate intervals along each dimension
for x_ind = 1:(len_x_ints-1)
    if X(1) <= x_ints(x_ind+1)
        break;
    end
end
for x_dot_ind = 1:(len_x_dot_ints-1)
    if X(2) <= x_dot_ints(x_dot_ind+1)
        break;
    end
end
for theta_ind = 1:(len_theta_ints-1)
    if X(3) <= theta_ints(theta_ind+1)
        break;
    end
end
for theta_dot_ind = 1:(len_theta_dot_ints-1)
    if X(4) <= theta_dot_ints(theta_dot_ind+1)
        break;
    end
end

%calculate final state index
s_ind = (x_ind-1)*(len_x_dot_ints-1)*(len_theta_ints-1)*(len_theta_dot_ints-1) + (x_dot_ind-1)*(len_theta_ints-1)*(len_theta_dot_ints-1) + (theta_ind-1)*(len_theta_dot_ints-1) + theta_dot_ind;
